<?php

namespace Drupal\sqrl\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\sqrl\Sqrl;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the SQRL unlink controller.
 */
class Unlink implements ContainerInjectionInterface {

  /**
   * @var \Drupal\sqrl\Sqrl
   */
  protected $sqrl;

  /**
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Link constructor.
   *
   * @param \Drupal\sqrl\Sqrl $sqrl
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   */
  public function __construct(Sqrl $sqrl, AccountProxyInterface $current_user) {
    $this->sqrl = $sqrl;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): Unlink {
    return new static(
      $container->get('sqrl.handler'),
      $container->get('current_user')
    );
  }

  /**
   * @param \Drupal\user\UserInterface $user
   *
   * @return \Drupal\Core\Access\AccessResult
   */
  public function access(UserInterface $user): AccessResult {
    if ($this->currentUser->isAuthenticated() && $this->currentUser->id() === $user->id()) {
      return AccessResult::allowed();
    }
    return AccessResult::forbidden();
  }

  /**
   * @return array
   */
  public function request(): array {
    return [
      'sqrl' => $this->sqrl->buildMarkup('unlink'),
    ];
  }

}
