<?php

namespace Drupal\sqrl;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\sqrl\Entity\IdentityInterface;
use Drupal\user\UserInterface;

class Identities {

  /**
   * @var string
   */
  protected $idk;

  /**
   * @var string
   */
  protected $pidk;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Identities constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * @param $idk
   *
   * @return \Drupal\sqrl\Identities
   */
  public function setIdk($idk): Identities {
    $this->idk = $idk;
    return $this;
  }

  /**
   * @param $pidk
   *
   * @return \Drupal\sqrl\Identities
   */
  public function setPidk($pidk): Identities {
    $this->pidk = $pidk;
    return $this;
  }

  /**
   * @param string $key
   * @param int|null $uid
   *
   * @return \Drupal\sqrl\Entity\IdentityInterface|null
   */
  protected function getIdentity($key, $uid = NULL): ?IdentityInterface {
    if (empty($key)) {
      return NULL;
    }
    $conditions = [
      'idk' => $key,
    ];
    if ($uid !== NULL) {
      $conditions['user'] = $uid;
    }
    try {
      /** @var \Drupal\sqrl\Entity\IdentityInterface[] $identities */
      $identities = $this->entityTypeManager->getStorage('sqrl_identity')->loadByProperties($conditions);
    }
    catch (InvalidPluginDefinitionException $e) {
    }
    catch (PluginNotFoundException $e) {
    }
    if (!empty($identities)) {
      return reset($identities);
    }
    return NULL;
  }

  /**
   * @param int $uid
   *
   * @return \Drupal\sqrl\Entity\IdentityInterface|null
   */
  public function getIdentities($uid): ?IdentityInterface {
    try {
      /** @var \Drupal\sqrl\Entity\IdentityInterface[] $identities */
      $identities = $this->entityTypeManager->getStorage('sqrl_identity')->loadByProperties([
        'user' => $uid,
      ]);
    }
    catch (InvalidPluginDefinitionException $e) {
    }
    catch (PluginNotFoundException $e) {
    }
    if (!empty($identities)) {
      return reset($identities);
    }
    return NULL;
  }

  /**
   * @param int@null $uid
   *
   * @return \Drupal\sqrl\Entity\IdentityInterface|null
   */
  public function getIdentityByIdk($uid = NULL): ?IdentityInterface {
    return $this->getIdentity($this->idk, $uid);
  }

  /**
   * @param int|null $uid
   *
   * @return \Drupal\sqrl\Entity\IdentityInterface|null
   */
  public function getIdentityByPidk($uid = NULL): ?IdentityInterface {
    return $this->getIdentity($this->pidk, $uid);
  }

  /**
   * @param \Drupal\user\UserInterface $account
   */
  public function cancel(UserInterface $account): void {
    /** @var \Drupal\sqrl\Entity\IdentityInterface[] $identities */
    $identities = $this->getIdentities($account->id());
    foreach ($identities as $identity) {
      try {
        if ($identity->removeUser($account)) {
          $identity->save();
        }
      }
      catch (EntityStorageException $e) {
      }
    }
  }

  /**
   * @param int $uid
   *
   * @return bool
   */
  public function hasUserEnabledIdentities($uid): bool {
    try {
      return (bool) $this->entityTypeManager->getStorage('sqrl_identity')->getQuery()
        ->condition('user', $uid)
        ->condition('status', 1)
        ->notExists('sid')
        ->count()
        ->execute();
    }
    catch (InvalidPluginDefinitionException $e) {
    }
    catch (PluginNotFoundException $e) {
    }
    return FALSE;
  }

}
